#include "tpager.h"
/* Atoms */
char *root_atom_name[] = {
	"_NET_NUMBER_OF_DESKTOPS",
	"_NET_CURRENT_DESKTOP",
	"_NET_DESKTOP_NAMES",
	"_NET_SHOWING_DESKTOP"
};
#define ROOT_ATOM_COUNT 4

#define NUMBER_OF_DESKTOPS root_atom[0]
#define CURRENT_DESKTOP root_atom[1]
#define DESKTOP_NAMES root_atom[2]
#define SHOWING_DESKTOP root_atom[3]
Atom root_atom[ROOT_ATOM_COUNT];

void root_props_init(Display *disp) {
	XInternAtoms (disp, root_atom_name, ROOT_ATOM_COUNT, True, root_atom);
}

/**
 * Counts the number of desktops
 */
int count_workspc(Display *disp, Window root) {
	return get_prop_cardinal(disp, root, NUMBER_OF_DESKTOPS);
}

/**
 * Gets current workspace, on a 0 index base
 */
int curr_workspc(Display *disp, Window root) {
	return get_prop_cardinal(disp, root, CURRENT_DESKTOP);
}

void set_curr_workspc(Display *disp, Window root, int workspc) {
	XEvent xev;

	xev.xclient.type = ClientMessage;
	xev.xclient.serial = 0;
	xev.xclient.send_event = True;
	xev.xclient.display = disp;
	xev.xclient.window = root;
	xev.xclient.message_type = CURRENT_DESKTOP;
	xev.xclient.format = 32;
	xev.xclient.data.l[0] = workspc;
	xev.xclient.data.l[1] = 0;
	xev.xclient.data.l[2] = 0;

	XSendEvent (disp,
	root,
	False,
	SubstructureRedirectMask | SubstructureNotifyMask,
	&xev);
}

void set_workspc_count(Display *disp, Window root, int count) {
	XEvent xev;

	xev.xclient.type = ClientMessage;
	xev.xclient.serial = 0;
	xev.xclient.window = root;
	xev.xclient.send_event = True;
	xev.xclient.display = disp;
	xev.xclient.message_type = NUMBER_OF_DESKTOPS;
	xev.xclient.format = 32;
	xev.xclient.data.l[0] = count;

	XSendEvent (disp,
		root,
		False,
		SubstructureRedirectMask | SubstructureNotifyMask,
		&xev);
}

void show_desktop (Display *disp, Window root, int show) {
	XEvent xev;

	xev.xclient.type = ClientMessage;
	xev.xclient.serial = 0;
	xev.xclient.window = root;
	xev.xclient.send_event = True;
	xev.xclient.display = disp;
	xev.xclient.message_type = SHOWING_DESKTOP;
	xev.xclient.format = 32;
	xev.xclient.data.l[0] = show;

	XSendEvent (disp,
		root,
		False,
		SubstructureRedirectMask | SubstructureNotifyMask,
		&xev);
}

int is_showing_desktop (Display *disp, Window root) {
	return get_prop_cardinal(disp, root, SHOWING_DESKTOP);
}

void on_root_property_change(Atom a) {
	if ( a == CURRENT_DESKTOP ) {
		//update current desktop
		update_curr_workspc();
	} else if ( a == NUMBER_OF_DESKTOPS ) {
		//update number of desktops
		update_desktop_count();
	} else if (a == SHOWING_DESKTOP) {
		update_desktop_visibility();
	}
}

