#include "tpager.h"

typedef struct pager {
	//visual props
	int x;
	int y;
	int width;
	int height;
	int bw;
	int bh;
	Window win;
	GC gc;
	XFontStruct* xfs;
	//defines if desktops are drawn horizontaly or verticaly
	int horz;
	long border_c;
	long unselected_c;
	long selected_c;
	//pager props:
	//current workspace
	int curr;
	//number of workspaces
	int count;
	//showing desktop
	int showing;
}pager;

typedef struct xserv {
	Display *disp;
	//Window Manager aka root window
	Window wm;
	int screen;
	int width;
	int height;
	int depth;
}xserv;
/* Program constants */
#define BORDER_WIDTH 0
#define BG_COLOR 0xEEEEEE
#define BUTTON_NORMAL 0x999999
#define BUTTON_HIGHLIGHT 0xCCCCCC
#define PROGRAM_NAME "tpager"
/* Macros */
#define set_fg(color) XSetForeground(xs.disp,pg.gc,color);
#define set_bg(color) XSetBackground(xs.disp,pg.gc,color);
#define fill_rect(x,y,width,height) XFillRectangle(xs.disp,pg.win,pg.gc,x,y,width,height)
#define draw_line(x1,y1,x2,y2) XDrawLine(xs.disp,pg.win,pg.gc,x1,y1,x2,y2)

#define ungrab() XUngrabPointer(xs.disp, CurrentTime)
#define grab(mask, curs) (XGrabPointer(xs.disp, pg.win, False, mask, \
	GrabModeAsync, GrabModeAsync, None, curs, CurrentTime) == GrabSuccess)

#define CPARM(a,b) a = a < 0 ? (b) : a
xserv xs;
pager pg;
int text_y = 0;
int font_height;
int char_max_width = 0;
//
int refresh_req = 0;
xserv init_xs() {
	xserv *x;
	x = (xserv *)malloc(sizeof(xserv));
	x->disp = XOpenDisplay(NULL);
	x->screen = DefaultScreen(x->disp);
	x->width = DisplayWidth (x->disp,x->screen);
	x->height = DisplayHeight (x->disp, x->screen);
	x->wm = RootWindow( x->disp, x->screen);
	x->depth = DefaultDepth (x->disp,x->screen);
	//init client and root props
	root_props_init(x->disp);
	client_props_init(x->disp);
	return *x;
}

pager init_pager(xserv xsv, int x, int y, int w, int h, int horz, long b, long s, long n) {
	XSetWindowAttributes attributes;
	char *window_name = PROGRAM_NAME;
	XSizeHints hints;
	unsigned long attr_mask;
	char* font_name;
	XGCValues gc_vals;
	pager *p = (pager *)malloc(sizeof(pager));
	p->x = x;
	p->y = y;
	p->bw = w;
	p->height = h;
	p->bh = h - 2;
	p->curr = curr_workspc (xsv.disp, xsv.wm);
	p->count = count_workspc (xsv.disp, xsv.wm);
	p->width = w * p->count + 1;
	
	p->horz = horz;
	p->border_c = b;
	p->selected_c = s;
	p->unselected_c = n;
	//pg.win setup
	attributes.background_pixel = BG_COLOR;
	attributes.border_pixel = BlackPixel(xsv.disp, xsv.screen);
	attributes.event_mask = ButtonPressMask|ExposureMask;
	//Remove window frames from panel
	attributes.override_redirect = True;
	attr_mask = CWOverrideRedirect|CWBackPixel|CWBorderPixel|CWEventMask;
	//listen for properties change (where EMWH appear)
	XSelectInput (xsv.disp, xsv.wm, PropertyChangeMask );
	p->win = XCreateWindow(
		xsv.disp,
		xsv.wm,
		x,y,w * p->count + 1, h, 0,
		xsv.depth, InputOutput, DefaultVisual(xsv.disp, xsv.screen), 
		attr_mask, &attributes);
	//define window props
	set_panel_type (xsv.disp, p->win);
	set_panel_state (xsv.disp, p->win);
	set_workspc (xsv.disp, p->win, 0xFFFFFFFF);
	XStoreName (xsv.disp, p->win, window_name);
	
	XMapWindow (xsv.disp, p->win);
	XFlush (xsv.disp);
	//font related
	font_name = "-*-lucida*-m*-r-*-*-12-*-*";
	p->xfs = XLoadQueryFont (xsv.disp, font_name);
	font_height = p->xfs->ascent + p->xfs->descent;
	text_y = p->xfs->ascent + ((h - p->xfs->ascent) / 2);
	
	gc_vals.graphics_exposures = False;
	gc_vals.font = p->xfs->fid;
	p->gc = XCreateGC (xsv.disp, xsv.wm, GCFont |GCGraphicsExposures, &gc_vals);
	char_max_width = (p->xfs->max_bounds).width;
	return *p;
}

void draw_pager(void) {
	int i;
	int w = pg.count > 0 ? (pg.width/pg.count) : 0 ;
	int x = 1;
	set_fg(pg.border_c);
	fill_rect(0, 0, pg.width * pg.count + 1, pg.height);
	for (i = 0; i < pg.count; i++) {
		if (i == pg.curr) {
			set_fg(pg.selected_c);
		} else {
			set_fg(pg.unselected_c);
		}
		fill_rect(x, 1, pg.bw - 1, pg.bh);
		x += pg.bw;
		
	}
	XFlush(xs.disp);
}

void update_curr_workspc(void) {
	pg.curr = curr_workspc (xs.disp, xs.wm);
	draw_pager();
}
void update_desktop_count(void) {
	pg.count = count_workspc (xs.disp, xs.wm);
	pg.width = pg.count * pg.bw + 1;
	XResizeWindow (xs.disp, pg.win, pg.width, pg.height);
	draw_pager();
}

void update_desktop_visibility(void) {
	pg.showing = is_showing_desktop(xs.disp, xs.wm);
}

void on_click(int x, int y, int button) {
	if (button == 1 && pg.count > 1 && pg.width > 0) {
		set_curr_workspc(xs.disp, xs.wm, x * pg.count / pg.width);
	} else if (button == 3) {
		show_desktop(xs.disp, xs.wm, !pg.showing);
	} else if (button == 2) {
		//move pager
		XEvent ev;
		if (XGrabPointer(xs.disp, pg.win, False, ButtonPressMask|ButtonReleaseMask|PointerMotionMask, GrabModeAsync, GrabModeAsync, None, None, CurrentTime) != GrabSuccess) {
			return;
		}
		XGrabServer(xs.disp);
		do {
			XMaskEvent(xs.disp, ButtonPressMask|ButtonReleaseMask|PointerMotionMask, &ev);
			pg.x += ev.xbutton.x;
			pg.y += ev.xbutton.y;
			XMoveWindow(xs.disp, pg.win, pg.x, pg.y);
			draw_pager();
			
		} while (ev.type != ButtonRelease);
		XUngrabServer(xs.disp);
		XUngrabPointer(xs.disp, CurrentTime);
	}
}

void show_usage(char *prog) {
	printf("Usage: %s [-x <val>] [-y <val>] [-w <val>] [-h <val>] [-horz|-vert] [-b <color>] [-n <color>] [-s <color>] [-set <val>]\n", prog);
	printf("\t-x <val>\tX coordinate on screen (default: 0)\n");
	printf("\t-y <val>\tY coordinate on screen (default: bottom of screen)\n");
	printf("\t-width <val>\tWidth of each pager (default: 24)\n");
	printf("\t-height <val>\tHeight of pager (default: 24)\n");
	printf("\t-vert\tDisplays squares verticaly\n");
	printf("\t-horz\tDisplays squares horizontaly (default)\n");
	printf("\t-b <color>\tSets the border color (default: 0x000000)\n");
	printf("\t-s <color>\tSets selected color (default: 0xCCCCCC)\n");
	printf("\t-n <color>\tSets not selected color (default: 0x999999)\n");
	printf("\t-set <val>\tSets the number of virtual desktops (default: window manager's choice)\n");
	printf("Note: <color> arguments should be entered as an hexadecimal base value they are represented in 0xRRGGBB.\n");
}

int main (int argc, char *argv[]) {
	int i, x, y, w, h, horz, b, n, s, set;
	x = y = w = h = horz = b = n = s = set = -1;
	char *temp;
	XEvent event;
	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i],"-x") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			x = (int)strtol (argv[i + 1], &temp, 10);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-y") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			y = (int)strtol (argv[i + 1], &temp, 10);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-width") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			w = (int)strtol (argv[i + 1], &temp, 10);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-height") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			h = (int)strtol (argv[i + 1], &temp, 10);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-b") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			b = (int)strtol (argv[i + 1], &temp, 16);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-s") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			s = (int)strtol (argv[i + 1], &temp, 16);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-n") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			n = (int)strtol (argv[i + 1], &temp, 16);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-set") == 0) {
			if ( i + 1 >= argc ) {
				show_usage(argv[0]);
				return 1;
			}
			set = (int)strtol (argv[i + 1], &temp, 10);
			if (strcmp(temp,"") != 0) {
				show_usage(argv[0]);
				return 1;
			}
			i++;
		} else if (strcmp(argv[i],"-horz") == 0) {
			horz = 1;
		} else if (strcmp(argv[i],"-vert") == 0) {
			horz = 0;
		} else {
			show_usage (argv[0]);
			return 1;
		}
	}
	xs = init_xs();
	//Default values
	CPARM(w, 24);
	CPARM(h, 24);
	CPARM(x, 0);
	CPARM(y, xs.height - h);
	CPARM(horz, 1);
	CPARM(b, 0);
	CPARM(s, 0xCCCCCC);
	CPARM(n, 0x999999);
	pg = init_pager(xs, x, y, w, h, horz, b, s, n);
	if (set > 0) {
		set_workspc_count(xs.disp, xs.wm, set);
	}
	
	while(1) {
		XNextEvent( xs.disp, &event);
		switch(event.type) {
			case Expose:
				if( event.xexpose.window == pg.win)
					draw_pager();
				break;
			case PropertyNotify:
			on_root_property_change(event.xproperty.atom);
			break;
			case ButtonPress:
			if(event.xbutton.window == pg.win)
				on_click(event.xbutton.x, event.xbutton.y, event.xbutton.button);
			break;
			case ClientMessage:
			break;
		}
	}
	return 0;
}
